﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win;
using Infragistics.Win.UltraWinTabs;
using Infragistics.Win.UltraWinTabControl;

namespace NavigationButtons
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            for ( int i = 0; i < 100; i ++ )
            {
                string text = string.Format("Tab{0}", i + 1);
                UltraTab tab = this.tab.Tabs.Add(text, text);
            }

            CreationFilter filter = new CreationFilter();
            filter.ScrollButtonClick += new CreationFilter.ScrollButtonClickHandler(filter_ScrollButtonClick);
            this.tab.CreationFilter = filter;

        }

        void filter_ScrollButtonClick(ArrowDirection direction)
        {
            UltraTabControlAction action =
                direction == ArrowDirection.Left ?
                UltraTabControlAction.NavigatePreviousTab :
                UltraTabControlAction.NavigateNextTab;

            this.tab.PerformAction(action);
        }


        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        /// <param name="disposing">true if managed resources should be disposed; otherwise, false.</param>
        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (components != null)
                    components.Dispose();

                CreationFilter filter = this.tab.CreationFilter as CreationFilter;
                if ( filter != null )
                    filter.Dispose();
            }
            base.Dispose(disposing);
        }
    }

    public class CreationFilter : IUIElementCreationFilter, IDisposable
    {
        private const int BUTTON_WIDTH = 20;
        private ScrollButtonUIElement left = null;
        private ScrollButtonUIElement right = null;
        public delegate void ScrollButtonClickHandler(ArrowDirection direction);
        public event ScrollButtonClickHandler ScrollButtonClick;

        void IUIElementCreationFilter.AfterCreateChildElements(UIElement parent)
        {
            TabRowUIElement tabRowElement = parent as TabRowUIElement;
            if ( tabRowElement != null )
            {
                foreach ( UIElement childElement in tabRowElement.ChildElements )
                {
                    TabItemUIElement tabElement = childElement as TabItemUIElement;
                    if ( tabElement != null )
                    {
                        Rectangle rect = tabElement.Rect;
                        rect.X += BUTTON_WIDTH;
                        tabElement.Rect = rect;
                    }
                }
            }

            TabHeaderAreaUIElement headerAreaElement = parent as TabHeaderAreaUIElement;
            if ( headerAreaElement != null )
            {
                TabScrollButtonUIElement scrollButton =
                headerAreaElement.GetDescendant(typeof(TabScrollButtonUIElement))
                as TabScrollButtonUIElement;

                if ( scrollButton != null &&
                     scrollButton.Parent == headerAreaElement )
                {
                    headerAreaElement.ChildElements.Remove(scrollButton);

                    //  Left
                    Rectangle bounds = headerAreaElement.RectInsideBorders;
                    bounds.Width = BUTTON_WIDTH;
                    ScrollButtonUIElement left = this.GetScrollButton(ArrowDirection.Left, headerAreaElement);
                    left.Rect = bounds;
                    headerAreaElement.ChildElements.Add(left);


                    //  Right
                    bounds = headerAreaElement.RectInsideBorders;
                    bounds.X = bounds.Right - BUTTON_WIDTH;
                    bounds.Width = BUTTON_WIDTH;
                    ScrollButtonUIElement right = this.GetScrollButton(ArrowDirection.Right, headerAreaElement);
                    right.Rect = bounds;
                    headerAreaElement.ChildElements.Add(right);
                }
            }
        }

        bool IUIElementCreationFilter.BeforeCreateChildElements(UIElement parent)
        {
            return false;
        }

        private ScrollButtonUIElement GetScrollButton(ArrowDirection direction, UIElement parent)
        {
            ScrollButtonUIElement button = null;

            switch ( direction )
            {
                case ArrowDirection.Left:
                    if ( this.left == null )
                    {
                        this.left = new ScrollButtonUIElement(parent, ScrollButton.Left, 0);
                        this.left.ElementClick += this.OnElementClick;
                    }
                    button = this.left;
                    break;

                case ArrowDirection.Right:
                    if ( this.right == null )
                    {
                        this.right = new ScrollButtonUIElement(parent, ScrollButton.Right, 0);
                        this.right.ElementClick += this.OnElementClick;
                    }
                    button = this.right;
                    break;
            }

            return button;
        }

        private void OnElementClick(object sender, UIElementEventArgs e)
        {
            ArrowDirection direction =
                e.Element == this.left ?
                ArrowDirection.Left :
                ArrowDirection.Right;

            if ( this.ScrollButtonClick != null )
                this.ScrollButtonClick(direction);
        }

        void IDisposable.Dispose()
        {
            this.Dispose();
        }

        public void Dispose()
        {
            if ( this.left != null )
            {
                this.left.ElementClick -= this.OnElementClick;
                this.left = null;
            }

            if ( this.right != null )
            {
                this.right.ElementClick -= this.OnElementClick;
                this.right = null;
            }
        }
    }
}
